// #autoload
// #name = FilterMsg
// #version = 3.11
// #date = October 21, 2002
// #author = xgalaxy / Halide
// #warrior = Halide
// #email = halide@pandoranetworks.org
// #email = xgalaxy@home.com
// #web = http://halide.tribes-universe.com/
// #description = Filters death messages from chat.
// #include = support/file_tools.cs
// #include = support/player_support.cs
// #include = support/callback.cs
// #include = support/mute_tools.cs
// #readme = FilterMsg/Readme.txt
// #config = FilterMsgOptions

if(!isObject(FilterMsg))
{
	new ScriptObject(FilterMsg)
	{
		class = FilterMsg;
		version = "3.11";
		currentLine = 0;
		// prefs
		nameSize = 90;		// width for player names
		tagSize = 30;		// width for player clan tags
		padding = 6;		// text padding inside hud (even number)
		hideToggle = false;	// hide when all huds are toggled?
	};
}

function FilterMsgToggle(%val)
{
	if(!%val)
		return;

	FilterMsg.enabled = !FilterMsg.enabled;

	alxPlay(sButtonOver, 0, 0, 0);

	if(FilterMsg.enabled)
	{
		FilterMsgHud.setVisible(FilterMsg.isVisible);
		addMessageHudLine("\c2FilterMsg enabled.");
	}
	else
	{
		FilterMsgHud.setVisible(false);
		addMessageHudLine("\c2FilterMsg disabled.");
	}
}

// START FilterMsg package
package	FilterMsg
{

	function FilterMsg::setup(%this)
	{
		if(%this.setup)
			return;

		// search and load *.ico.cs files in the FilterMsg folder
		%search = "FilterMsg/*.ico.cs";

		for(%fileName = findFirstFile(%search); %fileName !$= ""; %fileName = findNextFile(%search))
			exec(%fileName);

		// set default prefs before loading them, to prevent missing prefs; create the hud
		%this.setDefaults();

		// load prefs file
		exec("prefs/FilterPrefs.txt");

		// set the prefs on the options gui
		%this.setPrefs();

		// these are for cleaning up the hud at the end of each mission, there could be more that I could add here
		callback.add(onUserClientDrop,			 	"FilterMsg.cleanup();");
		callback.add(CallbackMsgGameOver,		 	"FilterMsg.cleanup();");

		// these tell the script what to do when we get certain kill messages...

		// compmod
		callback.add(CallbackMsgRearshotKill,		"FilterMsg.onDeathMessage");

		// tr2
		callback.add(CallbackMsgHotPotato,			"FilterMsg.onDeathMessage");
		callback.add(CallbackMsgTR2Knockdown,		"FilterMsg.onDeathMessage");
		callback.add(CallbackMsgGrid,				"FilterMsg.onDeathMessage");

		// normal
		callback.add(CallbackMsgLegitKill,			"FilterMsg.onDeathMessage");
		callback.add(CallbackMsgTeamKill,			"FilterMsg.onDeathMessage");
		callback.add(CallbackMsgHeadShotKill,		"FilterMsg.onDeathMessage");

		// self
		callback.add(CallbackMsgSuicide,			"FilterMsg.onDeathMessage");
		callback.add(CallbackMsgSelfKill,			"FilterMsg.onDeathMessage");
		callback.add(CallbackMsgTurretSelfKill,		"FilterMsg.onDeathMessage");

		// vehicle
		callback.add(CallbackMsgVehicleKill,		"FilterMsg.onDeathMessage");
		callback.add(CallbackMsgVehicleSpawnKill,	"FilterMsg.onDeathMessage");
		callback.add(CallbackMsgVehicleCrash,		"FilterMsg.onDeathMessage");

		// turret
		callback.add(CallbackMsgTurretKill,			"FilterMsg.onDeathMessage");
		callback.add(CallbackMsgCTurretKill,		"FilterMsg.onDeathMessage");

		// other
		callback.add(CallbackMsgOOBKill,			"FilterMsg.onDeathMessage");
		callback.add(CallbackMsgCampKill,			"FilterMsg.onDeathMessage");
		callback.add(CallbackMsgLightningKill,		"FilterMsg.onDeathMessage");
		callback.add(CallbackMsgLavaKill,			"FilterMsg.onDeathMessage");
		callback.add(CallbackMsgExplosionKill,		"FilterMsg.onDeathMessage");
		callback.add(CallbackMsgRogueMineKill,		"FilterMsg.onDeathMessage");

		%this.setup = true;
	}
	////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//// Preference loading, creating, and saving functions ////////////////////////////////////////////////////////
	function FilterMsg::setDefaults(%this)
	{
		// the preference file must not have been available or the user
		// specifically chose to reset to default option values

		%this.isVisible = true;		// hud is visible
		%this.isTrans = true;		// hud is transparent
		%this.colorNames = true;	// color code names
		%this.colorMe = true;		// color code your name
		%this.scrollDown = true;	// scroll hud lines down
		%this.timeOut = true;		// time out messages
		%this.muteDeathMsgs = true;	// mute death messages from chat hud
		%this.globalKills = true;	// show all kills
		%this.stripTags = true;		// strip clan tags from names
		%this.numMsgLines = 4;		// number of lines total in the hud
		%this.textAlign = 1;		// text alignment (0-3, Left-Left, Center-Center, Right-Left, Left-Right)
		%this.curSet = "Default";	// current icon set

		%this.fontColor[0] = "<color:33bbdd>";	// Highlight
		%this.fontColor[1] = "<color:11dd11>";	// Friendly
		%this.fontColor[2] = "<color:dd2222>";	// Enemy
		%this.fontColor[3] = "<color:bfbbbb>";	// Generic

		%this.enabled = true;			// enabled (for toggle)
		%this.msgTimeOutDelay = 10000;	// number of ms to keep a line in the hud before it is cleared

		// (re)build the hud
		%this.build();
	}

	function FilterMsg::newIconSet(%this, %name, %iconWidth, %iconHeight)
	{
		FltMsgIconSet.add(%name, 0);

		return %this.iconSet[%name] = new ScriptObject()
		{
			class = FilterMsgIcons;
			width = %iconWidth;
			height = %iconHeight;
		};
	}

	function FilterMsg::savePrefs(%this)
	{
		// write each line to a file...
		%this.file = new FileObject();
		%this.file.openForWrite("prefs/FilterPrefs.txt");
		%this.file.writeline("//==========================================================================");
		%this.file.writeline("//                  -= FilterMsg Preferences File =-");
		%this.file.writeline("//	         By: xgalaxy / Halide  Version: " @ %this.version @ ";");
		%this.file.writeline("//");
		%this.file.writeline("//    You may alter the settings in this file but it would be easier");
		%this.file.writeline("//    to do so in the in-game options page specified for FilterMsg.");
		%this.file.writeline("//");
		%this.file.writeline("//    Tribes 2 must be restarted for changes to take affect in this file.");
		%this.file.writeline("//==========================================================================");
		%this.file.writeline("// Settings");
		%this.file.writeline("//==========================================================================");
		%this.file.writeline(%this @ ".isVisible = "		@ %this.isVisible @ ";");
		%this.file.writeline(%this @ ".isTrans = "			@ %this.isTrans @ ";");
		%this.file.writeline(%this @ ".colorNames = "		@ %this.colorNames @ ";");
		%this.file.writeline(%this @ ".colorMe = " 			@ %this.colorMe @ ";");
		%this.file.writeline(%this @ ".scrollDown = "		@ %this.scrollDown @ ";");
		%this.file.writeline(%this @ ".timeOut = "			@ %this.timeOut @ ";");
		%this.file.writeline(%this @ ".muteDeathMsgs = "	@ %this.muteDeathMsgs @ ";");
		%this.file.writeline(%this @ ".globalKills = "		@ %this.globalKills @ ";");
		%this.file.writeline(%this @ ".stripTags = "		@ %this.stripTags @ ";");
		%this.file.writeline(%this @ ".numMsgLines = "		@ %this.numMsgLines @ ";");
		%this.file.writeline(%this @ ".textAlign = "		@ %this.textAlign @ ";");
		%this.file.writeline(%this @ ".curSet = \""			@ %this.curSet @ "\";");
		%this.file.writeline("//==========================================================================");
		%this.file.writeline("// These options not available in the options gui");
		%this.file.writeline("//==========================================================================");
		%this.file.writeline(%this @ ".enabled = "			@ %this.enabled @ ";");
		%this.file.writeline(%this @ ".msgTimeOutDelay = "	@ %this.msgTimeOutDelay @ ";");
		%this.file.writeline(%this @ ".fontColor[0] = \""	@ %this.fontColor[0] @ "\";");
		%this.file.writeline(%this @ ".fontColor[1] = \""	@ %this.fontColor[1] @ "\";");
		%this.file.writeline(%this @ ".fontColor[2] = \""	@ %this.fontColor[2] @ "\";");
		%this.file.writeline(%this @ ".fontColor[3] = \""	@ %this.fontColor[3] @ "\";");
		%this.file.writeline("//==========================================================================");

		// then close the file
		%this.file.close();
	}

	function FilterMsg::setPrefs(%this)
	{
		// make sure that the GUI is displaying the selected option values correctly
		// by forcing it to update ...
		FltMsgTransHud.setValue			(%this.isTrans);
		FltMsgVisibleHud.setValue		(%this.isVisible);
		FltMsgColorNames.setValue		(%this.colorNames);
		FltMsgColorMe.setValue			(%this.colorMe);
		FltMsgScrollDown.setValue		(%this.scrollDown);
		FltMsgTimeOut.setValue			(%this.timeOut);
		FltMsgMuteDeathMsgs.setValue	(%this.muteDeathMsgs);
		FltMsgGlobalKills.setValue		(%this.globalKills);
		FltMsgStripTags.setValue		(%this.stripTags);
		FltMsgNumMsgLinesSlider.setValue(%this.numMsgLines);
		FltMsgTextAlign.setSelected		(%this.textAlign);
		FltMsgIconSet.setValue			(%this.curSet);
	}

	/////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//// Team and Display Support Functions /////////////////////////////////////////////////////////////////////
	function FilterMsg::myTeam(%this)
	{
		// I wanted a simple function call instead of these things
		// so I put them in this stupid function, I'm lazy, sue me...
		%id = PlayerList.getMyID();
		%team = PlayerList.getTeamByID(%id);

		return %team;
	}

	function FilterMsg::evalTeam(%this, %name)
	{
		// Teams are either 0, 1, 2. 0 is always the observer team, but 1 and 2 change
		// depending on what team you are on. This function determines what team someone
		// is on as compared to your team and assigns 'color teams' to correspond to the
		// fontColor array
		%team = PlayerList.getTeamByName(%name);

			if(%this.myTeam() == 0)				// observer
				%team = 3;						// fontColor[3]
			else if(%this.myTeam() == %team)	// friendly
				%team = 1;						// fontColor[1]
			else								// enemy
				%team = 2;						// fontColor[2]

		return %team;
	}

	function FilterMsg::stripTag(%this, %name)
	{
		%pos1 = strStr(%name, "\c7");

		if(%pos1 > 0)
		{
			%pos2 = strStr(%name, "\c6");

			if(%pos2 < %pos1)
				return getSubStr(%name, %pos2 + 1, %pos1 - %pos2 - 1);
			else
			{
				%pos3 = strStr(%name, "\x11");
				return getSubStr(%name, %pos2 + 1, %pos3 - %pos2 - 1);
			}
		}
		else
		{
			%pos2 = strStr(%name, "\c8");
			%pos3 = strStr(%name, "\x11");
			return getSubStr(%name, %pos2 + 1, %pos3 - %pos2 - 1);
		}
	}

	function FilterMsg::colorByName(%this, %name)
	{
		// This function gives colors to the names based on their team
		// as compared to your team

		if(%name $= PlayerList.getMyName() && %this.colorMe)
			return %this.fontColor[0];

		if(%this.colorNames)
			return %this.fontColor[%this.evalTeam(%name)];

		return "";
	}

	function FilterMsg::cleanup(%this)
	{
		for(%i = 0; %i <= %this.numMsgLines; %i++)
			%this.moveLines();

		%this.currentLine = 0;

		for(%i = %this.scheduleNum; %i > 0; %i--)
		{
			if(isEventPending(%this.scheduleObj[%i]))
				cancel(%this.scheduleObj[%i]);
			else
				break;
		}

		%this.scheduleNum = 0;
	}

	function FilterMsg::moveLines(%this)
	{
		// this is a little tricky...
		// for each line (except the first line which is 0)
		// make the previous line the value of the current line
		if(%this.currentLine <= 1)
		{
			// this happens if there is only 1 printed line
			// and the schedule was triggered
			FilterMsgHud.line[0].setValue("");
			FilterMsgHud.icon[0].setBitmap("");
			FilterMsgHud.iconTK[0].setVisible(false);
		}
		else
		{
			for(%i = 1; %i < %this.currentLine; %i++)
			{
				FilterMsgHud.line[%i - 1].setValue(FilterMsgHud.line[%i].getValue());
				FilterMsgHud.icon[%i - 1].setBitmap(FilterMsgHud.icon[%i].bitmap);
				FilterMsgHud.iconTK[%i - 1].setVisible(FilterMsgHud.iconTK[%i].visible);

				// do this or end up with repeating lines
				FilterMsgHud.line[%i].setValue("");
				FilterMsgHud.icon[%i].setBitmap("");
				FilterMsgHud.iconTK[%i].setVisible(false);
			}
		}

		// don't decrement the current print line if its not greater then 0 or we end up with negative numbers :p
		// we do this because what we in effect did was delete the first line and move every line up one
		// so we have to change the current print line to reflect this
		if(%this.currentLine > 0)
			%this.currentLine--;
	}

	// the guts of the program! (called for every death, eek!)
	function FilterMsg::onDeathMessage(%this, %msgType, %msgString, %victimName, %victimGender, %victimPoss, %killerName, %killerGender, %killerPoss, %damageType)
	{
		if(!%this.enabled)
			return;

		// take off the pesky tags
		%victimName = detag(%victimName);
		%killerName = detag(%killerName);
		%msgType = detag(%msgType);

		// global kills? if not, stop here
		if(!%this.globalKills && !(%victimName $= PlayerList.getMyName() || %killerName $= PlayerList.getMyName()))
			return;

		// color names by team
		%victimColor = %this.colorByName(%victimName);
		%killerColor = %this.colorByName(%killerName);

		// strip clan tags?
		if(%this.stripTags)
		{
			%victimName = %this.stripTag(%victimName);
			%killerName = %this.stripTag(%killerName);
		}

		// messages are formatted nicely here with clipping...
		if((%killerName !$= "") && (%killerName !$= %victimName))
			%message = "<lmargin:" @ %this.lmargin[0] @ ">" @
						"<clip:" @ %this.clip[0] @ ">" @
						"<just:" @ %this.align[0] @ ">" @
						%killerColor @ %killerName @
						"</clip>" @
						"<lmargin:" @ %this.lmargin[1] @ ">" @
						"<clip:" @ %this.clip[1] @ ">" @
						"<just:" @ %this.align[1] @ ">" @
						%victimColor @ %victimName @
						"</clip>";
		else
			%message = "<lmargin:" @ %this.lmargin[1] @ ">" @
						"<clip:" @ %this.clip[1] @ ">" @
						"<just:" @ %this.align[1] @ ">" @
						%victimColor @ %victimName @
						"</clip>";

		if(%msgType $= "MsgHeadshotKill")
			%damageType = "headshot";

		// adds a line to the hud; params: text line, icon file, tk? (true/false)
		%this.addHudLine(%message, %this.iconSet[%this.curSet].icon[%damageType], %msgType $= "MsgTeamKill");

		// return mute is part of the mute_tools.cs script I wrote for support back in the day
		// this in effect just mutes the output to the defaultMessageCallback function
		// -xgalaxy
		if(%this.muteDeathMsgs)
			return mute;
	}

	function FilterMsg::addHudLine(%this, %message, %icon, %tk)
	{
		// should we remove and move lines?
		if(%this.currentLine == %this.numMsgLines)
			%this.moveLines();
		else
			if(%this.timeOut)
				%this.scheduleObj[%this.scheduleNum++] = %this.schedule(%this.msgTimeOutDelay, "moveLines");

		FilterMsgHud.line[%this.currentLine].setValue(%message);
		FilterMsgHud.icon[%this.currentLine].setBitmap("FilterMsgIcons/" @ %icon);
		FilterMsgHud.iconTK[%this.currentLine].setVisible(%tk);

		// advance the current line position
		%this.currentLine++;
	}

	function FilterMsg::build(%this)
	{
		%this.marginSize = %this.nameSize + (%this.stripTags ? 0 : %this.tagSize);

		// there should only be one instance of the object
		if(isObject(FilterMsgHud))
		{
			// clear the hud and cancel time-out schedules
			%this.cleanup();
			// delete the old hud
			FilterMsgHud.delete();
		}

		if($hudPosFilterMsgHud $= "")
		{
			// default position, bottom-left of screen
			%xpos = 8;
			%ypos = getWord($pref::Video::Resolution, 1) - 75 - (%this.numMsgLines * %this.iconSet[%this.curSet].height);
		}
		else
		{
			%xpos = getWord($hudPosFilterMsgHud, 0);
			%ypos = getWord($hudPosFilterMsgHud, 1);
		}

		%xext = (%this.marginSize * 2) + %this.iconSet[%this.curSet].width;
		%yext = (%this.numMsgLines * (%this.iconSet[%this.curSet].height + 1)) + %this.padding;

		switch(%this.textAlign)
		{
			case 0:
				%this.align[0] = "left";
				%this.align[1] = "left";
				%this.clip[0] = %this.marginSize - %this.padding;
				%this.clip[1] = %this.marginSize - %this.padding;
				%this.lmargin[0] = %this.padding;
				%this.lmargin[1] = %this.padding + %xext - %this.marginSize;
			case 1:
				%this.align[0] = "center";
				%this.align[1] = "center";
				%this.clip[0] = %this.marginSize;
				%this.clip[1] = %this.marginSize;
				%this.lmargin[0] = 0;
				%this.lmargin[1] = %xext - %this.marginSize;
			case 2:
				%this.align[0] = "right";
				%this.align[1] = "left";
				%this.clip[0] = %this.marginSize - %this.padding;
				%this.clip[1] = %this.marginSize - %this.padding;
				%this.lmargin[0] = 0;
				%this.lmargin[1] = %this.padding + %xext - %this.marginSize;
			case 3:
				%this.align[0] = "left";
				%this.align[1] = "right";
				%this.clip[0] = %this.marginSize - %this.padding;
				%this.clip[1] = %this.marginSize - 0;
				%this.lmargin[0] = %this.padding;
				%this.lmargin[1] = %xext - %this.marginSize - %this.padding;
		}

		new GuiControl(FilterMsgHud)
		{
			profile = "GuiDefaultProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = %xpos SPC %ypos;
			extent = %xext SPC %yext;
			minExtent = "8 8";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			helpTag = "0";

			new ShellFieldCtrl(FilterMsgHudBack)
			{
				profile = "GuiChatBackProfile";
				horizSizing = "right";
				vertSizing = "bottom";
				position = "0 0";			// if they have HudMover this SHOULD get over-ridden...
				extent = %xext SPC %yext;	// width x height, was originally 276 72 before cosmetic changes
				minExtent = "8 8";
			};
		};
		PlayGui.add(FilterMsgHud);

		FilterMsgHudBack.setVisible(!%this.isTrans);

		// a sexy way of dynamically creating the lines in the hud
		for(%i = 0; %i < %this.numMsgLines; %i++)
			%this.addLine(%i);
	}

	function FilterMsg::addLine(%this, %i)
	{
		%xext = (240 + %this.iconSet[%this.curSet].width);
		%yext = %this.iconSet[%this.curSet].height;
		%xpos = 0;
		%ypos = (%i * (%this.iconSet[%this.curSet].height + 1)) + ((%this.iconSet[%this.curSet].height - 10) / 2);

		%id = %this.scrollDown ? %this.numMsgLines - %i - 1 : %i;

		FilterMsgHud.line[%id] = new GuiMLTextCtrl()
		{
			profile = "GuiTextObjHudLeftProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = %xpos SPC %ypos;
			extent = %xext SPC %yext;
			minExtent = "8 8";
			visible = "1";
		};
		FilterMsgHud.add(FilterMsgHud.line[%id]);

		%xpos = ((getWord(FilterMsgHud.extent, 0) / 2) - (%this.iconSet[%this.curSet].width / 2));
		%ypos = (%i * (%this.iconSet[%this.curSet].height + 1)) + (%this.padding / 2);
		%xext = %this.iconSet[%this.curSet].width;
		%yext = %this.iconSet[%this.curSet].height;

		FilterMsgHud.icon[%id] = new GuiBitmapCtrl()
		{
			profile = "GuiDefaultProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = %xpos SPC %ypos;
			extent = %xext SPC %yext;
			minExtent = "8 8";
			visible = "1";
			hideCursor = "0";
			bypassHideCursor = "0";
			helpTag = "0";
			wrap = "0";
		};
		FilterMsgHud.add(FilterMsgHud.icon[%id]);

		FilterMsgHud.iconTK[%id] = new GuiBitmapCtrl()
		{
			profile = "GuiDefaultProfile";
			horizSizing = "right";
			vertSizing = "bottom";
			position = %xpos SPC %ypos;
			extent = %xext SPC %yext;
			minExtent = "8 8";
			visible = "0";
			hideCursor = "0";
			bypassHideCursor = "0";
			helpTag = "0";
			wrap = "0";
		};
		FilterMsgHud.iconTK[%id].setBitmap("FilterMsgIcons/" @ %this.iconSet[%this.curSet].icon["tk"]);
		FilterMsgHud.add(FilterMsgHud.iconTK[%id]);
	}

	//////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//// Options Menu ////////////////////////////////////////////////////////////////////////////////////////////
	function FilterMsg::buildOptions(%this)
	{
		if(isObject(FilterMsgOptions))
			return;

		exec("gui/FilterMsgOptions.gui");

		FltMsgTextAlign.add("Left - Left", 0);
		FltMsgTextAlign.add("Center - Center", 1);
		FltMsgTextAlign.add("Right - Left", 2);
		FltMsgTextAlign.add("Left - Right", 3);
	}

	function quit()
	{
		FilterMsg.savePrefs();

		parent::quit();
	}

	function OptionsDlg::onWake(%this)
	{
		if(!FilterMsg.bind)
		{
			$RemapName[$RemapCount] = "FilterMsg Toggle";
			$RemapCmd[$RemapCount] = "FilterMsgToggle";
			$RemapCount++;

			FilterMsg.bind = true;
		}

		parent::onWake(%this);
	}

	function clientCmdTogglePlayHuds(%val)
	{
		parent::clientCmdTogglePlayHuds(%val);
		
		if(FilterMsg.hideToggle && FilterMsg.isVisible && FilterMsg.enabled)
			FilterMsgHud.setVisible(%val);
	}

	function LoadingGui::onWake(%this)
	{
		// hudmover support
		if(isObject(HM) && isObject(HudMover))
			hudmover::addhud(FilterMsgHud, "FilterMsg Hud");

		parent::onWake(%this);
	}

	function demoPlaybackComplete()
	{
		parent::demoPlaybackComplete();

		FilterMsg.cleanup();
	}

	function OptionsDlg::applyGraphicChanges(%this)
	{
		parent::applyGraphicChanges(%this);

		FilterMsg.build();
	}

	function lobbyLeaveGame()
	{
		parent::lobbyLeaveGame();

		FilterMsg.savePrefs();
	}

// END FilterMsg package
};
// activate the package and let us get things rolling... :D
activatePackage(FilterMsg);

// need this here for Writer Support
FilterMsg.buildOptions();
// activate & initialize!
FilterMsg.setup();
